import React, { useEffect, useRef, useState, useImperativeHandle } from 'react';
import SpinBox from "@/components/Spin/SpinBox";

import { ScrollView } from '@tarojs/components';
import Taro from '@tarojs/taro';

export default React.forwardRef((props, ref) => {
  const {
    render,
    request,
    params = {},
    pageSize = 10,
    onError,
    onDataChange,
    noData,
    className,
    ...leftProps
  } = props
  const [loading, setLoading] = useState(false)

  const contextRef = useRef()
  const loadingRef = useRef(false)
  const [payload, setPayload] = useState(params)
  const [list, setList] = useState([])
  const pageRef = useRef({ current: 1, pages: 0 })
  const hasMore = pageRef.current.current < pageRef.current.pages

  // 滚动
  const handleScrollToLower = (e) => {
    const loading = loadingRef.current

    if (!loading && hasMore) {
      setPayload({
        ...payload,
        pageNum: pageRef.current.current + 1
      })
    }
  }

  const fetchList = (params) => {
    if (!request) return;

    setLoading(true)
    loadingRef.current = true
    request(params).then((res) => {
      const { records, ...pageInfo } = res || {}
      const lst = pageInfo.current === 1 ? records || [] : list.concat(records || [])
      setList(lst)
      if (onDataChange) {
        onDataChange(lst)
      }

      pageRef.current = pageInfo
      loadingRef.current = false

      setLoading(false)

    }).catch((err) => {
      loadingRef.current = false
      console.error(err)
      setLoading(false)

      if (onError) {
        onError(err)
      }
    })
  }

  const fetchRef = useRef()
  fetchRef.current = fetchList

  // 联动状态, 设置查询参数
  useEffect(() => {
    setPayload({
      ...params || {},
      pageNum: 1,
      pageSize,
    })
  }, [pageSize, params])

  // 请求数据
  useEffect(() => {
    fetchRef.current(payload)
  }, [payload, request])

  useEffect(() => {
    Taro.nextTick(() => {
      Taro.createSelectorQuery().select('.list-view').node(function (res) {
        contextRef.current = res.node
      }).exec()
    })
  }, [])

  // https://developers.weixin.qq.com/miniprogram/dev/api/ui/scroll/ScrollViewContext.html
  // ScrollViewContext 透传给父组件
  //https://zh-hans.reactjs.org/docs/hooks-reference.html#useimperativehandle
  useImperativeHandle(ref, () => ({
    context: contextRef.current,
  }))

  return (

    <ScrollView
      scrollY
      enhanced
      onScrollToLower={handleScrollToLower}
      {...leftProps}
      className={`${className} list-view`}
    >
      <SpinBox loading={loading} className='index-container' >

        {!render
          ? props.children
          : list.map((item, index) => render({ item, index }))
        }
        {!list || !list.length && noData}

        {list && list.length > 0 && !hasMore &&

          <view className='botton'>这是我的底线</view>

        }
      </SpinBox>

    </ScrollView>
  )
})
